#include "daq_config.h"

#include <stdio.h>
#include <stdarg.h>

#include "nds_logging.h"

#define DEFAULT_VERBOSE_DEBUGGING 0

struct nds_log_ {
  int	s_group_mask;
  int	s_group_debug_level[ NDS_LOG_GROUP_SIZE_MAX ];
};

typedef struct nds_log_ nds_log_t;

static nds_log_t
log_info = {
  /* --------------------------------------------------------------------
   * Set bit mask of things that should be debugged by
   * default.
   * ----------------------------------------------------------------- */
#if DEFAULT_VERBOSE_DEBUGGING
  ( 1 << NDS_LOG_GROUP_CONNECTION )
  | ( 1 << NDS_LOG_GROUP_VERBOSE_ERRORS )
  | ( 1 << NDS_LOG_GROUP_TRACE_ENTRY_EXIT )
  | ( 1 << NDS_LOG_GROUP_STATUS_UPDATE )
#else	/* DEFAULT_VERBOSE_DEBUGGING */
  0
#endif /* DEFAULT_VERBOSE_DEBUGGING */
  ,
  /* --------------------------------------------------------------------
   * Setting of the default logging levels
   * ----------------------------------------------------------------- */
  {
    30,	/* NDS_LOG_GROUP_CONNECTION */
    30, /* NDS_LOG_GROUP_VERBOSE_ERRORS */
    20, /* NDS_LOG_GROUP_TRACE_ENTRY_EXIT */
    20, /* NDS_LOG_GROUP_STATUS_UPDATE */
    20, /* NDS_LOG_GROUP_USER */
  }

};


int
nds_logging_check( int Group, int Level )
{
  int retval = 0;

  if ( ( ( 1 << Group ) & log_info.s_group_mask )
       && ( log_info.s_group_debug_level[ Group ] >= Level ) )
  {
    retval = 1;
  }

  return retval;
}

void
nds_logging_enable( int Group )
{
  if ( ( Group < 0 )
       || ( Group >= NDS_LOG_GROUP_SIZE_MAX ) )
  {
    return;
  }
  log_info.s_group_mask |= ( 1<<Group );
  fprintf( stderr, "DEBUG: Mask: %d\n", log_info.s_group_mask );
}

void
nds_logging_disable( int Group )
{
  if ( ( Group < 0 )
       || ( Group >= NDS_LOG_GROUP_SIZE_MAX ) )
  {
    return;
  }
  log_info.s_group_mask &= ~( 1<<Group );
  fprintf( stderr, "DEBUG: Mask: %d\n", log_info.s_group_mask );
}

void
nds_logging_debug_level( int Group, int Level )
{
  if ( ( Group < 0 )
       || ( Group >= NDS_LOG_GROUP_SIZE_MAX ) )
  {
    return;
  }
  log_info.s_group_debug_level[ Group ] = Level;
}

void
nds_logging_flush( )
{
  fflush( stderr );
}

void
nds_logging_printf( const char* MessageFormat, ... )
{
   char buf[2048];
    va_list	ap;

    va_start( ap, MessageFormat );

#if HAVE_VSNPRINTF_S
    vsnprintf_s
#else /* HAVE_VSNPRINTF_S */
    vsnprintf
#endif /* HAVE_VSNPRINTF_S */
    ( buf, sizeof( buf ),
#if HAVE_VSNPRINTF_S
      sizeof( buf ) - 1,
#endif /* HAVE_VSNPRINTF_S */
      MessageFormat, ap );

    fprintf( stderr, "%s", buf );

    va_end( ap );
}
